-----------------------------------------------------------------------
--                              Mast                                 --
--     Modelling and Analysis Suite for Real-Time Applications       --
--                                                                   --
--                       Copyright (C) 2000-2009                     --
--                 Universidad de Cantabria, SPAIN                   --
--                                                                   --
-- Authors: Michael Gonzalez       mgh@unican.es                     --
--          Jose Javier Gutierrez  gutierjj@unican.es                --
--          Jose Carlos Palencia   palencij@unican.es                --
--          Jose Maria Drake       drakej@unican.es                  --
--          Maria Cue              cuem@unican.es                    --
--          Juan Maria Rivas	   rivasjm@unican.es                 --
--                                                                   --
-- This program is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This program is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this program; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-----------------------------------------------------------------------

with Mast.Processing_Resources,
     Mast.HOSPA_Parameters,
     Mast.Annealing_Parameters,
     Mast.Linear_Translation,
     Priority_Queues,Hash_Lists,List_Exceptions,
     Mast.Tools.Schedulability_Index,
     Mast.Scheduling_Parameters,Mast.Tool_Exceptions,
     Mast.Scheduling_Policies,Mast.Tools;

with Ada.Text_IO,Ada.Integer_Text_IO,
     Ada.Numerics.Discrete_Random,Ada.Numerics.Float_Random,
     Ada.Numerics.Generic_Elementary_Functions,
     Mast.Max_Numbers,Ada.Float_Text_IO;

use type Mast.Tools.Schedulability_Index.Index;
use Ada.Text_IO,Ada.Integer_Text_IO,Ada.Float_Text_IO;

with Mast.linear_global_deadline_assignment_tools,
     Mast.linear_local_deadline_assignment_tools;

package body Mast.Linear_Deadline_Assignment_Tools is

   ATC_Enabled : constant Boolean:=False;
   Debug : constant Boolean := False;
   My_Verbose : constant Boolean := False;

   type Processor_ID is new Natural;
   type Transaction_ID is new Natural;
   type Task_ID is new Natural;


   ----------------------
   -- Global Deadlines --
   ----------------------

   procedure Global_HOSDA
     (The_System : in out MAST.Systems.System;
      The_Tool : in Mast.Tools.Worst_Case_Analysis_Tool;
      Verbose : in Boolean:=False)
   is
   begin
      Mast.linear_global_deadline_assignment_tools.Global_HOSDA(The_System,
                                                                The_tool,
                                                                Verbose);
   end Global_HOSDA;


   procedure Proportional_Global_Deadline_Assignment
     (The_System : in out MAST.Systems.System;
      The_Tool : in Mast.Tools.Worst_Case_Analysis_Tool;
      Verbose : in Boolean:=False)
   is
   begin
      Mast.linear_global_deadline_assignment_tools.
        Proportional_Global_Deadline_Assignment(The_System,The_tool,Verbose);
   end Proportional_Global_Deadline_Assignment;


   procedure Normalized_Global_Deadline_Assignment
     (The_System : in out MAST.Systems.System;
      The_Tool : in Mast.Tools.Worst_Case_Analysis_Tool;
      Verbose : in Boolean:=False)
   is
   begin
      Mast.linear_global_deadline_assignment_tools.
        normalized_Global_Deadline_Assignment(The_System,The_tool,Verbose);
   end Normalized_Global_Deadline_Assignment;


   ---------------------
   -- Local Deadlines --
   ---------------------

   procedure Local_HOSDA
     (The_System : in out MAST.Systems.System;
      The_Tool : in Mast.Tools.Worst_Case_Analysis_Tool;
      Verbose : in Boolean:=False)
   is
   begin
      Mast.linear_local_deadline_assignment_tools.Local_HOSDA (The_System,
                                                               The_Tool,
                                                               Verbose);
   end Local_HOSDA;

   procedure Proportional_Local_Deadline_Assignment
     (The_System : in out MAST.Systems.System;
      The_Tool : in Mast.Tools.Worst_Case_Analysis_Tool;
      Verbose : in Boolean:=False)
   is
   begin
      Mast.linear_local_deadline_assignment_tools.
        Proportional_Local_Deadline_Assignment (The_System,The_Tool,Verbose);
   end Proportional_Local_Deadline_Assignment;

   procedure Normalized_Local_Deadline_Assignment
     (The_System : in out MAST.Systems.System;
      The_Tool : in Mast.Tools.Worst_Case_Analysis_Tool;
      Verbose : in Boolean:=False)
   is
   begin
      Mast.linear_local_deadline_assignment_tools.
        Normalized_Local_Deadline_Assignment (The_System,The_Tool,Verbose);
   end Normalized_Local_Deadline_Assignment;
end Mast.Linear_Deadline_Assignment_Tools;

